#include  "idl_otbfastmatchingimagefilter.h"

#include "otbImage.h"
#include "itkBinaryThresholdImageFilter.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkCurvatureAnisotropicDiffusionImageFilter.h"
#include "itkGradientMagnitudeRecursiveGaussianImageFilter.h"
#include "itkSigmoidImageFilter.h"
#include "itkFastMarchingImageFilter.h"
#include "itkImageFileWriter.h"

ORFEOANDIDL_API void otbfastmatchingimagefilter(int argc, IDL_VPTR argv[], char *argk)
{
  static char inputFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(outputFilename, argv[1]->value.str.s);
  
  IDL_VPTR idlSeedX         = IDL_CvtDbl(1, &argv[2]);  
  IDL_VPTR idlSeedY         = IDL_CvtDbl(1, &argv[3]);  
  IDL_VPTR idlSigma         = IDL_CvtDbl(1, &argv[4]);  
  IDL_VPTR idlSigmoideAlpha = IDL_CvtDbl(1, &argv[5]);  
  IDL_VPTR idlSigmoideBeta  = IDL_CvtDbl(1, &argv[6]);  
  IDL_VPTR idlTimeThreshold = IDL_CvtDbl(1, &argv[7]);  
  IDL_VPTR idlStoppingValue = IDL_CvtDbl(1, &argv[8]);  
	
  int SeedX            = (unsigned int)idlSeedX->value.d;
  int SeedY            = (unsigned int)idlSeedY->value.d;
  double Sigma         = (double)idlSigma->value.d;
  double SigmoideAlpha = (double)idlSigmoideAlpha->value.d;
  double SigmoideBeta  = (double)idlSigmoideBeta->value.d;
  double TimeThreshold = (double)idlTimeThreshold->value.d;
  double StoppingValue = (double)idlStoppingValue->value.d;
  
  typedef float                                                 InternalPixelType;
  const   unsigned int                                          Dimension = 2;
  typedef unsigned char                                         OutputPixelType;
  typedef otb::Image< InternalPixelType, Dimension >            InternalImageType;
  typedef otb::Image< OutputPixelType, Dimension >              OutputImageType;
  typedef itk::BinaryThresholdImageFilter< InternalImageType, 
                                           OutputImageType    > ThresholdingFilterType;
  typedef  otb::ImageFileReader< InternalImageType >            ReaderType;
  typedef  otb::ImageFileWriter<  OutputImageType  >            WriterType;

  ThresholdingFilterType::Pointer thresholder   = ThresholdingFilterType::New();
  const InternalPixelType         timeThreshold = TimeThreshold; 
  ReaderType::Pointer             reader        = ReaderType::New();
  WriterType::Pointer             writer        = WriterType::New();

  thresholder->SetLowerThreshold(           0.0  );
  thresholder->SetUpperThreshold( timeThreshold  );
  thresholder->SetOutsideValue(  0  );
  thresholder->SetInsideValue(  255 );

  
  reader->SetFileName( inputFilename );
  writer->SetFileName( outputFilename );

    
  typedef itk::RescaleIntensityImageFilter< 
                               InternalImageType, 
                               OutputImageType >   CastFilterType;

  typedef   itk::CurvatureAnisotropicDiffusionImageFilter< 
                               InternalImageType, 
                               InternalImageType >  SmoothingFilterType;
 
  SmoothingFilterType::Pointer smoothing = SmoothingFilterType::New();
 
  typedef   itk::GradientMagnitudeRecursiveGaussianImageFilter< 
                               InternalImageType, 
                               InternalImageType >  GradientFilterType;

  typedef   itk::SigmoidImageFilter<                               
                               InternalImageType, 
                               InternalImageType >  SigmoidFilterType;
 
  GradientFilterType::Pointer  gradientMagnitude = GradientFilterType::New();
  SigmoidFilterType::Pointer sigmoid = SigmoidFilterType::New();
 
  sigmoid->SetOutputMinimum(  0.0  );
  sigmoid->SetOutputMaximum(  1.0  );
  
  typedef  itk::FastMarchingImageFilter< InternalImageType, 
                              InternalImageType >    FastMarchingFilterType;
 
  FastMarchingFilterType::Pointer  fastMarching = FastMarchingFilterType::New();
 
  smoothing->SetInput( reader->GetOutput() );
  gradientMagnitude->SetInput( smoothing->GetOutput() );
  sigmoid->SetInput( gradientMagnitude->GetOutput() );
  fastMarching->SetInput( sigmoid->GetOutput() );
  thresholder->SetInput( fastMarching->GetOutput() );
  writer->SetInput( thresholder->GetOutput() );

  smoothing->SetTimeStep( 0.125 );
  smoothing->SetNumberOfIterations(  10 );
  smoothing->SetConductanceParameter( 2.0 );
 
  gradientMagnitude->SetSigma(  Sigma  );
  
  const double alpha =  SigmoideAlpha;
  const double beta  =  SigmoideBeta;

  sigmoid->SetAlpha( alpha );
  sigmoid->SetBeta(  beta  );

  typedef FastMarchingFilterType::NodeContainer           NodeContainer;
  typedef FastMarchingFilterType::NodeType                NodeType;
  NodeContainer::Pointer seeds = NodeContainer::New();

  InternalImageType::IndexType  seedPosition;
  
  seedPosition[0] = SeedX;
  seedPosition[1] = SeedY;

  NodeType node;
  const double seedValue = 0.0;
  
  node.SetValue( seedValue );
  node.SetIndex( seedPosition );
 
  seeds->Initialize();
  seeds->InsertElement( 0, node );
 
  fastMarching->SetTrialPoints(  seeds  );
  
  CastFilterType::Pointer caster1 = CastFilterType::New();
  CastFilterType::Pointer caster2 = CastFilterType::New();
  CastFilterType::Pointer caster3 = CastFilterType::New();
  CastFilterType::Pointer caster4 = CastFilterType::New();

  WriterType::Pointer writer1 = WriterType::New();
  WriterType::Pointer writer2 = WriterType::New();
  WriterType::Pointer writer3 = WriterType::New();
  WriterType::Pointer writer4 = WriterType::New();

  caster1->SetInput( smoothing->GetOutput() );
  writer1->SetInput( caster1->GetOutput() );
  writer1->SetFileName("FastMarchingFilterOutput1.png");
  caster1->SetOutputMinimum(   0 );
  caster1->SetOutputMaximum( 255 );
  writer1->Update();

  caster2->SetInput( gradientMagnitude->GetOutput() );
  writer2->SetInput( caster2->GetOutput() );
  writer2->SetFileName("FastMarchingFilterOutput2.png");
  caster2->SetOutputMinimum(   0 );
  caster2->SetOutputMaximum( 255 );
  writer2->Update();

  caster3->SetInput( sigmoid->GetOutput() );
  writer3->SetInput( caster3->GetOutput() );
  writer3->SetFileName("FastMarchingFilterOutput3.png");
  caster3->SetOutputMinimum(   0 );
  caster3->SetOutputMaximum( 255 );
  writer3->Update();

  caster4->SetInput( fastMarching->GetOutput() );
  writer4->SetInput( caster4->GetOutput() );
  writer4->SetFileName("FastMarchingFilterOutput4.png");
  caster4->SetOutputMinimum(   0 );
  caster4->SetOutputMaximum( 255 );

  fastMarching->SetOutputSize( reader->GetOutput()->GetBufferedRegion().GetSize() );
  
  const double stoppingTime = StoppingValue;
  
  fastMarching->SetStoppingValue(  stoppingTime  );
  
  writer->Update();
  writer4->Update();
  

 
  typedef itk::ImageFileWriter< InternalImageType > InternalWriterType;

  InternalWriterType::Pointer mapWriter = InternalWriterType::New();
  mapWriter->SetInput( fastMarching->GetOutput() );
  mapWriter->SetFileName("FastMarchingFilterOutput4.mha");
  mapWriter->Update();

  InternalWriterType::Pointer speedWriter = InternalWriterType::New();
  speedWriter->SetInput( sigmoid->GetOutput() );
  speedWriter->SetFileName("FastMarchingFilterOutput3.mha");
  speedWriter->Update();

  InternalWriterType::Pointer gradientWriter = InternalWriterType::New();
  gradientWriter->SetInput( gradientMagnitude->GetOutput() );
  gradientWriter->SetFileName("FastMarchingFilterOutput2.mha");
  gradientWriter->Update();

}
