#include "idl_otborthorectification.h"

#include "otbMapProjections.h"
#include "otbStreamingImageFileWriter.h"
#include "otbPerBandVectorImageFilter.h"
#include "otbOrthoRectificationFilter.h"
#include "otbImageFileReader.h"
#include "otbStreamingImageFileWriter.h"
#include "otbVectorImage.h"

ORFEOANDIDL_API void otborthorectification(int argc, IDL_VPTR argv[], char *argk)
{
  static char              inputFilename[MAX_LENGTH];
  static char              outputFilename[MAX_LENGTH];
  static unsigned int      utmZone= 1;
  static char              hemisphere[MAX_LENGTH];
  static char              xGroundUL[MAX_LENGTH];
  static char              yGroundUL[MAX_LENGTH];
  static double            xGroundSampling = 1.;
  static double            yGroundSampling = 1.;
  static long unsigned int xSize=500;
  static long unsigned int ySize=500;
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(outputFilename, argv[1]->value.str.s);

  if (argv[2]->value.d != NULL)
    utmZone = static_cast<unsigned int>(argv[2]->value.d);

  if (argv[3]->value.str.s != NULL)
    strcpy(hemisphere, argv[3]->value.str.s); 

  if (argv[4]->value.str.s != NULL)
    strcpy(xGroundUL, argv[4]->value.str.s);
  
  if (argv[5]->value.str.s != NULL)
    strcpy(yGroundUL, argv[5]->value.str.s);
 
  if (argv[6]->value.d != NULL)
    xSize = static_cast<long unsigned int>(argv[6]->value.d);
  
  if (argv[7]->value.d != NULL)
    ySize = static_cast<long unsigned int>(argv[7]->value.d);

   if (argv[8]->value.d != NULL)
    xGroundSampling = argv[8]->value.d;
  
  if (argv[9]->value.d != NULL)
    yGroundSampling = argv[9]->value.d;

  typedef otb::Image<unsigned char, 2>                    CharImageType;
  typedef otb::Image<unsigned int, 2>                     ImageType;
  typedef otb::VectorImage<unsigned int, 2>               VectorImageType;
  typedef otb::ImageFileReader<VectorImageType>           ReaderType;
  typedef otb::StreamingImageFileWriter<VectorImageType>  WriterType;
  
  ReaderType::Pointer     	reader=ReaderType::New();
  WriterType::Pointer	    	writer=WriterType::New();
  
  reader->SetFileName(inputFilename);
  writer->SetFileName(outputFilename);
  
  
  typedef otb::UtmInverseProjection                                                 utmMapProjectionType ;
  typedef otb::OrthoRectificationFilter<ImageType, ImageType, utmMapProjectionType> OrthoRectifFilterType ;
  typedef otb::PerBandVectorImageFilter<VectorImageType, 
                                        VectorImageType, 
                                        OrthoRectifFilterType>                      PerBandFilterType;

  OrthoRectifFilterType::Pointer  orthoRectifFilter = OrthoRectifFilterType::New();
  
  utmMapProjectionType::Pointer utmMapProjection = utmMapProjectionType::New();
  utmMapProjection->SetZone(utmZone);
  utmMapProjection->SetHemisphere(*(hemisphere));
  orthoRectifFilter->SetMapProjection(utmMapProjection);

 
  PerBandFilterType::Pointer perBandFilter=PerBandFilterType::New();
  perBandFilter->SetFilter(orthoRectifFilter);
  perBandFilter->SetInput(reader->GetOutput());
    
  ImageType::IndexType start;
  start[0]=0;
  start[1]=0;
  orthoRectifFilter->SetOutputStartIndex(start);
				
  ImageType::SizeType size;
  size[0]=xSize;
  size[1]=ySize;
  orthoRectifFilter->SetSize(size);
				
  ImageType::SpacingType spacing;
  spacing[0]=xGroundSampling;
  spacing[1]=yGroundSampling;
  orthoRectifFilter->SetOutputSpacing(spacing);
				
  ImageType::PointType origin;
  origin[0]=strtod(xGroundUL, NULL);
  origin[1]=strtod(yGroundUL, NULL);
  orthoRectifFilter->SetOutputOrigin(origin);
 
  writer->SetInput(perBandFilter->GetOutput());		
  writer->SetTilingStreamDivisions();
  writer->Update();
}


