#include "idl_otbsvmimageclassification.h"

#include "otbImage.h"
#include "otbImageFileReader.h"
#include "otbSVMModel.h"
#include "otbSVMClassifier.h"
#include "itkImageToListAdaptor.h"
#include "itkSize.h"
#include "itkImageRegion.h"
#include "itkImageRegionIterator.h"
#include "itkRescaleIntensityImageFilter.h"
#include "otbImageFileWriter.h"


ORFEOANDIDL_API void otbsvmimageclassification(int argc, IDL_VPTR argv[], char *argk )
{
  namespace stat = itk::Statistics ;
  
  if (argc != 3)
    {
      std::cout << "Usage : " << argv[0] << " inputImage outputImage modelFile " 
                << std::endl ;
      return ;
    }
  
  static char imageFilename[MAX_LENGTH];
  static char modelFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(imageFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(modelFilename, argv[1]->value.str.s);
  
  if (argv[2]->value.str.s != NULL)
    strcpy(outputFilename, argv[2]->value.str.s);
  
  typedef double                          PixelType;
  typedef std::vector<PixelType>          VectorType;
  typedef int                             LabelPixelType; 
  const   unsigned int        	         Dimension = 2;

  typedef otb::Image< itk::FixedArray<PixelType,3>, Dimension >	InputImageType;
  typedef otb::ImageFileReader< InputImageType  >               ReaderType;
  typedef itk::Statistics::ImageToListAdaptor< InputImageType > SampleType;
  typedef otb::SVMModel< PixelType, LabelPixelType >            ModelType;
  typedef otb::SVMClassifier< SampleType, LabelPixelType >      ClassifierType ;

  ReaderType::Pointer     reader     = ReaderType::New();
  SampleType::Pointer     sample     = SampleType::New();
  ModelType::Pointer      model      = ModelType::New();
  ClassifierType::Pointer classifier = ClassifierType::New() ;


  reader->SetFileName( imageFilename  );
  reader->Update();  
  sample->SetImage(reader->GetOutput());
  model->LoadModel( modelFilename );
   
  int numberOfClasses = model->GetNumberOfClasses();    
  classifier->SetNumberOfClasses(numberOfClasses) ;
  classifier->SetModel( model );
  classifier->SetSample(sample.GetPointer()) ;
  classifier->Update() ;

  
  typedef ClassifierType::ClassLabelType	     OutputPixelType;
  typedef otb::Image< OutputPixelType, Dimension >   OutputImageType;
  typedef itk::Index<Dimension>                      myIndexType;
  typedef itk::Size<Dimension>                       mySizeType;
  typedef itk::ImageRegion<Dimension>                myRegionType;
  typedef itk::ImageRegionIterator< OutputImageType> OutputIteratorType;

  OutputImageType::Pointer outputImage = OutputImageType::New();

  mySizeType size;
  size[0] = reader->GetOutput()->GetRequestedRegion().GetSize()[0];
  size[1] = reader->GetOutput()->GetRequestedRegion().GetSize()[1];
  
  myIndexType start;
  start[0] = 0;
  start[1] = 0;
  
  myRegionType region;
  region.SetIndex( start );
  region.SetSize( size );
  
  outputImage->SetRegions( region );
  outputImage->Allocate();
  
  
  ClassifierType::OutputType*               membershipSample = classifier->GetOutput() ;
  ClassifierType::OutputType::ConstIterator m_iter           = membershipSample->Begin() ;
  ClassifierType::OutputType::ConstIterator m_last           = membershipSample->End() ;

  OutputIteratorType  outIt( outputImage, outputImage->GetBufferedRegion() );
  outIt.GoToBegin();
  
  while (m_iter != m_last && !outIt.IsAtEnd())
    {
      outIt.Set(m_iter.GetClassLabel());
      ++m_iter ;
      ++outIt;
    }
  
  
  typedef otb::Image< unsigned char, Dimension >                             FileImageType; 
  typedef otb::ImageFileWriter< FileImageType >                              WriterType;
  typedef itk::RescaleIntensityImageFilter< OutputImageType, FileImageType > RescalerType;
  
  RescalerType::Pointer rescaler = RescalerType::New();
  WriterType::Pointer   writer   = WriterType::New();

  rescaler->SetOutputMinimum( itk::NumericTraits< unsigned char >::min());
  rescaler->SetOutputMaximum( itk::NumericTraits< unsigned char >::max());

  rescaler->SetInput( outputImage );  
   
  writer->SetFileName( outputFilename  );
  writer->SetInput( rescaler->GetOutput() );
  
  writer->Update();
  
}
